package io.gitlab.jfronny.muscript.ast.dynamic.unpack;

import io.gitlab.jfronny.muscript.compiler.ExprWriter;
import io.gitlab.jfronny.muscript.data.Scope;
import io.gitlab.jfronny.muscript.data.dynamic.DynamicTypeConversionException;
import io.gitlab.jfronny.muscript.ast.DynamicExpr;
import io.gitlab.jfronny.muscript.ast.NumberExpr;
import io.gitlab.jfronny.muscript.annotations.CanThrow;
import io.gitlab.jfronny.muscript.annotations.UncheckedDynamic;
import io.gitlab.jfronny.muscript.ast.dynamic.DynamicCoerce;

import java.io.IOException;

@CanThrow
@UncheckedDynamic
public class NumberUnpack extends NumberExpr {
    public final DynamicExpr inner;

    public NumberUnpack(int chStart, int chEnd, DynamicExpr inner) {
        super(inner.order, chStart, chEnd);
        this.inner = inner;
    }

    @Override
    public Double get(Scope dataRoot) {
        try {
            return inner.get(dataRoot).asNumber().getValue();
        } catch (DynamicTypeConversionException e) {
            throw e.locational(chStart, chEnd);
        }
    }

    @Override
    public NumberExpr optimize() {
        DynamicExpr inner = this.inner.optimize();
        if (inner instanceof DynamicCoerce coerce) return coerce.inner.asNumberExpr();
        return new NumberUnpack(chStart, chEnd, inner.optimize());
    }

    @Override
    public void decompile(ExprWriter writer) throws IOException {
        inner.decompile(writer);
    }

    @Override
    public boolean equals(Object obj) {
        return obj instanceof NumberUnpack unpack && inner.equals(unpack.inner);
    }
}
